@@ -5,7 +5,7 @@ use crate::value::Value;
 use lazy_static::lazy_static;
 use std::collections::HashMap;
 
-type Function = fn(Vec<Value>) -> Value;
+type Function = fn(&[Value]) -> Value;
 
 pub struct Prototype {
     pub parameters: Vec<DataType>,
@@ -30,6 +30,7 @@ lazy_static! {
         map.insert("datalength", text_datalength);
         map.insert("char", text_char);
         map.insert("nchar", text_char);
+        map.insert("charindex", text_charindex);
         map.insert("replace", text_replace);
         map.insert("substring", text_substring);
         map.insert("stuff", text_stuff);
@@ -45,6 +46,9 @@ lazy_static! {
         map.insert("current_timestamp", date_current_timestamp);
         map.insert("now", date_current_timestamp);
         map.insert("makedate", date_make_date);
+        map.insert("maketime", date_make_time);
+        map.insert("dayname", date_dayname);
+        map.insert("monthname", date_monthname);
 
         // Numeric functions
         map.insert("abs", numeric_abs);
@@ -53,7 +57,13 @@ lazy_static! {
         map.insert("round", numeric_round);
         map.insert("square", numeric_square);
         map.insert("sin", numeric_sin);
+        map.insert("asin", numeric_asin);
         map.insert("cos", numeric_cos);
+        map.insert("acos", numeric_acos);
+        map.insert("tan", numeric_tan);
+        map.insert("atan", numeric_atan);
+        map.insert("atn2", numeric_atn2);
+        map.insert("sign", numeric_sign);
 
         // Other Functions
         map.insert("isnull", general_is_null);
@@ -165,6 +175,13 @@ lazy_static! {
                 result: DataType::Text,
             },
         );
+        map.insert(
+            "charindex",
+            Prototype {
+                parameters: vec![DataType::Text, DataType::Text],
+                result: DataType::Integer,
+            }
+        );
         map.insert(
             "replace",
             Prototype {
@@ -210,7 +227,7 @@ lazy_static! {
         map.insert(
             "concat",
             Prototype {
-                parameters: vec![DataType::Text, DataType::Text],
+                parameters: vec![DataType::Any, DataType::Any, DataType::Varargs(Box::new(DataType::Any))],
                 result: DataType::Text
              },
         );
@@ -258,6 +275,27 @@ lazy_static! {
                 result: DataType::Date,
             },
         );
+        map.insert(
+            "maketime",
+            Prototype {
+                parameters: vec![DataType::Integer, DataType::Integer, DataType::Integer],
+                result: DataType::Time,
+            },
+        );
+        map.insert(
+            "dayname",
+            Prototype {
+                parameters: vec![DataType::Date],
+                result: DataType::Text,
+            }
+        );
+        map.insert(
+            "monthname",
+            Prototype {
+                parameters: vec![DataType::Date],
+                result: DataType::Text,
+            }
+        );
         // Numeric functions
         map.insert(
             "abs",
@@ -301,6 +339,13 @@ lazy_static! {
                 result: DataType::Float,
             },
         );
+        map.insert(
+            "asin",
+            Prototype {
+                parameters: vec![DataType::Float],
+                result: DataType::Float,
+            },
+        );
         map.insert(
             "cos",
             Prototype {
@@ -308,6 +353,41 @@ lazy_static! {
                 result: DataType::Float,
             },
         );
+        map.insert(
+            "acos",
+            Prototype {
+                parameters: vec![DataType::Float],
+                result: DataType::Float,
+            },
+        );
+        map.insert(
+            "tan",
+            Prototype {
+                parameters: vec![DataType::Float],
+                result: DataType::Float,
+            },
+        );
+        map.insert(
+            "atan",
+            Prototype {
+                parameters: vec![DataType::Float],
+                result: DataType::Float,
+            },
+        );
+        map.insert(
+            "atn2",
+            Prototype {
+                parameters: vec![DataType::Float, DataType::Float],
+                result: DataType::Float,
+            },
+        );
+        map.insert(
+            "sign",
+            Prototype {
+                parameters: vec![DataType::Float],
+                result: DataType::Integer,
+            },
+        );
         // General functions
         map.insert(
             "isnull",
@@ -336,46 +416,46 @@ lazy_static! {
 
 // String functions
 
-fn text_lowercase(inputs: Vec<Value>) -> Value {
+fn text_lowercase(inputs: &[Value]) -> Value {
     Value::Text(inputs[0].as_text().to_lowercase())
 }
 
-fn text_uppercase(inputs: Vec<Value>) -> Value {
+fn text_uppercase(inputs: &[Value]) -> Value {
     Value::Text(inputs[0].as_text().to_uppercase())
 }
 
-fn text_reverse(inputs: Vec<Value>) -> Value {
+fn text_reverse(inputs: &[Value]) -> Value {
     Value::Text(inputs[0].as_text().chars().rev().collect::<String>())
 }
 
-fn text_replicate(inputs: Vec<Value>) -> Value {
+fn text_replicate(inputs: &[Value]) -> Value {
     let str = inputs[0].as_text();
     let count = inputs[1].as_int() as usize;
     Value::Text(str.repeat(count))
 }
 
-fn text_space(inputs: Vec<Value>) -> Value {
+fn text_space(inputs: &[Value]) -> Value {
     let n = inputs[0].as_int() as usize;
     Value::Text(" ".repeat(n))
 }
 
-fn text_trim(inputs: Vec<Value>) -> Value {
+fn text_trim(inputs: &[Value]) -> Value {
     Value::Text(inputs[0].as_text().trim().to_string())
 }
 
-fn text_left_trim(inputs: Vec<Value>) -> Value {
+fn text_left_trim(inputs: &[Value]) -> Value {
     Value::Text(inputs[0].as_text().trim_start().to_string())
 }
 
-fn text_right_trim(inputs: Vec<Value>) -> Value {
+fn text_right_trim(inputs: &[Value]) -> Value {
     Value::Text(inputs[0].as_text().trim_end().to_string())
 }
 
-fn text_len(inputs: Vec<Value>) -> Value {
+fn text_len(inputs: &[Value]) -> Value {
     Value::Integer(inputs[0].as_text().len() as i64)
 }
 
-fn text_ascii(inputs: Vec<Value>) -> Value {
+fn text_ascii(inputs: &[Value]) -> Value {
     let text = inputs[0].as_text();
     if text.is_empty() {
         return Value::Integer(0);
@@ -383,7 +463,7 @@ fn text_ascii(inputs: Vec<Value>) -> Value {
     Value::Integer(text.chars().next().unwrap() as i64)
 }
 
-fn text_left(inputs: Vec<Value>) -> Value {
+fn text_left(inputs: &[Value]) -> Value {
     let text = inputs[0].as_text();
     if text.is_empty() {
         return Value::Text("".to_string());
@@ -401,12 +481,12 @@ fn text_left(inputs: Vec<Value>) -> Value {
     Value::Text(substring)
 }
 
-fn text_datalength(inputs: Vec<Value>) -> Value {
+fn text_datalength(inputs: &[Value]) -> Value {
     let text = inputs[0].as_text();
     Value::Integer(text.as_bytes().len() as i64)
 }
 
-fn text_char(inputs: Vec<Value>) -> Value {
+fn text_char(inputs: &[Value]) -> Value {
     let code = inputs[0].as_int() as u32;
     if let Some(character) = char::from_u32(code) {
         return Value::Text(character.to_string());
@@ -414,7 +494,18 @@ fn text_char(inputs: Vec<Value>) -> Value {
     Value::Text("".to_string())
 }
 
-fn text_replace(inputs: Vec<Value>) -> Value {
+fn text_charindex(inputs: &[Value]) -> Value {
+    let substr = inputs[0].as_text();
+    let input = inputs[1].as_text();
+
+    if let Some(index) = input.to_lowercase().find(&substr.to_lowercase()) {
+        Value::Integer(index as i64 + 1)
+    } else {
+        Value::Integer(0)
+    }
+}
+
+fn text_replace(inputs: &[Value]) -> Value {
     let text = inputs[0].as_text();
     let old_string = inputs[1].as_text();
     let new_string = inputs[2].as_text();
@@ -434,7 +525,7 @@ fn text_replace(inputs: Vec<Value>) -> Value {
     Value::Text(result)
 }
 
-fn text_substring(inputs: Vec<Value>) -> Value {
+fn text_substring(inputs: &[Value]) -> Value {
     let text = inputs[0].as_text();
     //according to the specs, a stirng starts at position 1.
     //but in Rust, the index of a string starts from 0
@@ -451,7 +542,7 @@ fn text_substring(inputs: Vec<Value>) -> Value {
     Value::Text(text[start..(start + length as usize)].to_string())
 }
 
-fn text_stuff(inputs: Vec<Value>) -> Value {
+fn text_stuff(inputs: &[Value]) -> Value {
     let text = inputs[0].as_text();
     let start = (inputs[1].as_int() - 1) as usize;
     let length = inputs[2].as_int() as usize;
@@ -471,7 +562,7 @@ fn text_stuff(inputs: Vec<Value>) -> Value {
     Value::Text(text.into_iter().collect())
 }
 
-fn text_right(inputs: Vec<Value>) -> Value {
+fn text_right(inputs: &[Value]) -> Value {
     let text = inputs[0].as_text();
     if text.is_empty() {
         return Value::Text("".to_string());
@@ -486,7 +577,7 @@ fn text_right(inputs: Vec<Value>) -> Value {
     Value::Text(text[text.len() - number_of_chars..text.len()].to_string())
 }
 
-fn text_translate(inputs: Vec<Value>) -> Value {
+fn text_translate(inputs: &[Value]) -> Value {
     let mut text = inputs[0].as_text();
     let characters = inputs[1].as_text();
     let translations = inputs[2].as_text();
@@ -503,14 +594,14 @@ fn text_translate(inputs: Vec<Value>) -> Value {
     Value::Text(text)
 }
 
-fn text_unicode(inputs: Vec<Value>) -> Value {
+fn text_unicode(inputs: &[Value]) -> Value {
     if let Some(c) = inputs[0].as_text().chars().next() {
         return Value::Integer((c as u32).into());
     }
     Value::Integer(0)
 }
 
-fn text_soundex(inputs: Vec<Value>) -> Value {
+fn text_soundex(inputs: &[Value]) -> Value {
     let text = inputs[0].as_text();
     if text.is_empty() {
         return Value::Text("".to_string());
@@ -550,85 +641,141 @@ fn text_soundex(inputs: Vec<Value>) -> Value {
     Value::Text(result)
 }
 
-fn text_concat(inputs: Vec<Value>) -> Value {
-    let text: Vec<String> = inputs.iter().map(|v| v.as_text()).collect();
+fn text_concat(inputs: &[Value]) -> Value {
+    let text: Vec<String> = inputs.iter().map(|v| v.to_string()).collect();
     Value::Text(text.concat())
 }
 
 // Date functions
 
-fn date_current_date(_inputs: Vec<Value>) -> Value {
+fn date_current_date(_inputs: &[Value]) -> Value {
     let time_stamp = date_utils::get_unix_timestamp_ms();
     Value::Date(time_stamp)
 }
 
-fn date_current_time(_inputs: Vec<Value>) -> Value {
+fn date_current_time(_inputs: &[Value]) -> Value {
     let time_stamp = date_utils::get_unix_timestamp_ms();
     let time = date_utils::time_stamp_to_time(time_stamp);
     Value::Time(time)
 }
 
-fn date_current_timestamp(_inputs: Vec<Value>) -> Value {
+fn date_current_timestamp(_inputs: &[Value]) -> Value {
     let time_stamp = date_utils::get_unix_timestamp_ms();
     Value::DateTime(time_stamp)
 }
 
-fn date_make_date(inputs: Vec<Value>) -> Value {
+fn date_make_date(inputs: &[Value]) -> Value {
     let year = inputs[0].as_int() as i32;
     let day_of_year = inputs[1].as_int() as u32;
     let time_stamp = date_utils::time_stamp_from_year_and_day(year, day_of_year);
     Value::Date(time_stamp)
 }
 
+fn date_make_time(inputs: &[Value]) -> Value {
+    let hour = inputs[0].as_int();
+    let minute = inputs[1].as_int();
+    let second = inputs[2].as_int();
+    Value::Time(format!("{}:{:02}:{:02}", hour, minute, second))
+}
+
+fn date_dayname(inputs: &[Value]) -> Value {
+    let date = inputs[0].as_date();
+    let date_str = date_utils::date_to_day_name(date);
+    Value::Text(date_str)
+}
+
+fn date_monthname(inputs: &[Value]) -> Value {
+    let date = inputs[0].as_date();
+    let month_str = date_utils::date_to_month_name(date);
+    Value::Text(month_str)
+}
+
 // Numeric functions
 
-fn numeric_abs(inputs: Vec<Value>) -> Value {
+fn numeric_abs(inputs: &[Value]) -> Value {
     let value = inputs[0].as_int();
     Value::Integer(value.abs())
 }
 
-fn numeric_pi(_inputs: Vec<Value>) -> Value {
+fn numeric_pi(_inputs: &[Value]) -> Value {
     let pi = std::f64::consts::PI;
     Value::Float(pi)
 }
 
-fn numeric_floor(inputs: Vec<Value>) -> Value {
+fn numeric_floor(inputs: &[Value]) -> Value {
     let float_value = inputs[0].as_float();
     Value::Integer(float_value.floor() as i64)
 }
 
-fn numeric_round(inputs: Vec<Value>) -> Value {
+fn numeric_round(inputs: &[Value]) -> Value {
     let float_value = inputs[0].as_float();
     Value::Integer(float_value.round() as i64)
 }
 
-fn numeric_square(inputs: Vec<Value>) -> Value {
+fn numeric_square(inputs: &[Value]) -> Value {
     let int_value = inputs[0].as_int();
     Value::Integer(int_value * int_value)
 }
 
-fn numeric_sin(inputs: Vec<Value>) -> Value {
+fn numeric_sin(inputs: &[Value]) -> Value {
     let float_value = inputs[0].as_float();
     Value::Float(f64::sin(float_value))
 }
 
-fn numeric_cos(inputs: Vec<Value>) -> Value {
+fn numeric_asin(inputs: &[Value]) -> Value {
+    let float_value = inputs[0].as_float();
+    Value::Float(f64::asin(float_value))
+}
+
+fn numeric_cos(inputs: &[Value]) -> Value {
     let float_value = inputs[0].as_float();
     Value::Float(f64::cos(float_value))
 }
 
+fn numeric_acos(inputs: &[Value]) -> Value {
+    let float_value = inputs[0].as_float();
+    Value::Float(f64::acos(float_value))
+}
+
+fn numeric_tan(inputs: &[Value]) -> Value {
+    let float_value = inputs[0].as_float();
+    Value::Float(f64::tan(float_value))
+}
+
+fn numeric_atan(inputs: &[Value]) -> Value {
+    let float_value = inputs[0].as_float();
+    Value::Float(f64::atan(float_value))
+}
+
+fn numeric_atn2(inputs: &[Value]) -> Value {
+    let first = inputs[0].as_float();
+    let other = inputs[1].as_float();
+    Value::Float(f64::atan2(first, other))
+}
+
+fn numeric_sign(inputs: &[Value]) -> Value {
+    let float_value = inputs[0].as_float();
+    if float_value == 0.0 {
+        Value::Integer(0)
+    } else if float_value > 0.0 {
+        Value::Integer(1)
+    } else {
+        Value::Integer(-1)
+    }
+}
+
 // General functions
 
-fn general_is_null(inputs: Vec<Value>) -> Value {
+fn general_is_null(inputs: &[Value]) -> Value {
     Value::Boolean(inputs[0].data_type() == DataType::Null)
 }
 
-fn general_is_numeric(inputs: Vec<Value>) -> Value {
+fn general_is_numeric(inputs: &[Value]) -> Value {
     let input_type = inputs[0].data_type();
     Value::Boolean(input_type.is_number())
 }
 
-fn general_type_of(inputs: Vec<Value>) -> Value {
+fn general_type_of(inputs: &[Value]) -> Value {
     let input_type = inputs[0].data_type();
-    Value::Text(input_type.literal().to_string())
+    Value::Text(input_type.to_string())
 }
