@@ -12,13 +12,19 @@ pub enum StatementKind {
     OrderBy,
     GroupBy,
     AggregateFunction,
+    GlobalVariable,
 }
 
 pub trait Statement {
-    fn get_statement_kind(&self) -> StatementKind;
+    fn kind(&self) -> StatementKind;
     fn as_any(&self) -> &dyn Any;
 }
 
+pub enum Query {
+    Select(GQLQuery),
+    GlobalVariableDeclaration(GlobalVariableStatement),
+}
+
 pub struct GQLQuery {
     pub statements: HashMap<String, Box<dyn Statement>>,
     pub has_aggregation_function: bool,
@@ -39,7 +45,7 @@ impl Statement for SelectStatement {
         self
     }
 
-    fn get_statement_kind(&self) -> StatementKind {
+    fn kind(&self) -> StatementKind {
         StatementKind::Select
     }
 }
@@ -53,7 +59,7 @@ impl Statement for WhereStatement {
         self
     }
 
-    fn get_statement_kind(&self) -> StatementKind {
+    fn kind(&self) -> StatementKind {
         StatementKind::Where
     }
 }
@@ -67,7 +73,7 @@ impl Statement for HavingStatement {
         self
     }
 
-    fn get_statement_kind(&self) -> StatementKind {
+    fn kind(&self) -> StatementKind {
         StatementKind::Having
     }
 }
@@ -81,7 +87,7 @@ impl Statement for LimitStatement {
         self
     }
 
-    fn get_statement_kind(&self) -> StatementKind {
+    fn kind(&self) -> StatementKind {
         StatementKind::Limit
     }
 }
@@ -95,7 +101,7 @@ impl Statement for OffsetStatement {
         self
     }
 
-    fn get_statement_kind(&self) -> StatementKind {
+    fn kind(&self) -> StatementKind {
         StatementKind::Offset
     }
 }
@@ -116,7 +122,7 @@ impl Statement for OrderByStatement {
         self
     }
 
-    fn get_statement_kind(&self) -> StatementKind {
+    fn kind(&self) -> StatementKind {
         StatementKind::OrderBy
     }
 }
@@ -130,26 +136,41 @@ impl Statement for GroupByStatement {
         self
     }
 
-    fn get_statement_kind(&self) -> StatementKind {
+    fn kind(&self) -> StatementKind {
         StatementKind::GroupBy
     }
 }
 
-pub struct AggregateFunction {
-    pub function_name: String,
-    pub argument: String,
+pub enum AggregateValue {
+    Expression(Box<dyn Expression>),
+    Function(String, String),
 }
 
-pub struct AggregationFunctionsStatement {
-    pub aggregations: HashMap<String, AggregateFunction>,
+pub struct AggregationsStatement {
+    pub aggregations: HashMap<String, AggregateValue>,
 }
 
-impl Statement for AggregationFunctionsStatement {
+impl Statement for AggregationsStatement {
     fn as_any(&self) -> &dyn Any {
         self
     }
 
-    fn get_statement_kind(&self) -> StatementKind {
+    fn kind(&self) -> StatementKind {
         StatementKind::AggregateFunction
     }
 }
+
+pub struct GlobalVariableStatement {
+    pub name: String,
+    pub value: Box<dyn Expression>,
+}
+
+impl Statement for GlobalVariableStatement {
+    fn as_any(&self) -> &dyn Any {
+        self
+    }
+
+    fn kind(&self) -> StatementKind {
+        StatementKind::GlobalVariable
+    }
+}
