@@ -5,10 +5,13 @@ use std::hash::Hash;
 use std::hash::Hasher;
 use std::vec;
 
+use gitql_ast::environment::Environment;
 use gitql_ast::object::GQLObject;
 use gitql_ast::statement::GQLQuery;
+use gitql_ast::statement::Query;
 use gitql_ast::statement::SelectStatement;
 
+use crate::engine_executor::execute_global_variable_statement;
 use crate::engine_executor::execute_statement;
 
 const GQL_COMMANDS_IN_ORDER: [&str; 8] = [
@@ -22,15 +25,30 @@ const GQL_COMMANDS_IN_ORDER: [&str; 8] = [
     "limit",
 ];
 
-pub struct EvaluationValues {
-    pub groups: Vec<Vec<GQLObject>>,
-    pub hidden_selections: Vec<std::string::String>,
+pub enum EvaluationResult {
+    SelectedGroups(Vec<Vec<GQLObject>>, Vec<std::string::String>),
+    SetGlobalVariable,
 }
 
 pub fn evaluate(
-    repos: &Vec<git2::Repository>,
+    env: &mut Environment,
+    repos: &[gix::Repository],
+    query: Query,
+) -> Result<EvaluationResult, String> {
+    match query {
+        Query::Select(gql_query) => evaluate_select_query(env, repos, gql_query),
+        Query::GlobalVariableDeclaration(global_variable) => {
+            execute_global_variable_statement(env, &global_variable)?;
+            Ok(EvaluationResult::SetGlobalVariable)
+        }
+    }
+}
+
+pub fn evaluate_select_query(
+    env: &mut Environment,
+    repos: &[gix::Repository],
     query: GQLQuery,
-) -> Result<EvaluationValues, String> {
+) -> Result<EvaluationResult, String> {
     let mut groups: Vec<Vec<GQLObject>> = Vec::new();
     let mut alias_table: HashMap<String, String> = HashMap::new();
 
@@ -53,18 +71,26 @@ pub fn evaluate(
                     // If table name is empty no need to perform it on each repository
                     if select_statement.table_name.is_empty() {
                         execute_statement(
+                            env,
                             statement,
                             &repos[0],
                             &mut groups,
                             &mut alias_table,
                             &hidden_selections,
                         )?;
+
+                        // If the main group is empty, no need to perform other statements
+                        if groups.is_empty() || groups[0].is_empty() {
+                            return Ok(EvaluationResult::SelectedGroups(vec![], hidden_selections));
+                        }
+
                         continue;
                     }
 
                     // If table name is not empty, must perform it on each repository
                     for repo in repos {
                         execute_statement(
+                            env,
                             statement,
                             repo,
                             &mut groups,
@@ -75,10 +101,7 @@ pub fn evaluate(
 
                     // If the main group is empty, no need to perform other statements
                     if groups.is_empty() || groups[0].is_empty() {
-                        return Ok(EvaluationValues {
-                            groups: vec![],
-                            hidden_selections,
-                        });
+                        return Ok(EvaluationResult::SelectedGroups(vec![], hidden_selections));
                     }
 
                     // If Select statement has table name and distinct flag, keep only unique values
@@ -87,8 +110,9 @@ pub fn evaluate(
                     }
                 }
                 _ => {
-                    // Any other statement can be performend on first or non repository
+                    // Any other statement can be performed on first or non repository
                     execute_statement(
+                        env,
                         statement,
                         first_repo,
                         &mut groups,
@@ -119,10 +143,10 @@ pub fn evaluate(
     }
 
     // Return the groups and hidden selections to be used later in GUI or TUI ...etc
-    Ok(EvaluationValues {
-        groups: groups.to_owned(),
+    Ok(EvaluationResult::SelectedGroups(
+        groups.to_owned(),
         hidden_selections,
-    })
+    ))
 }
 
 fn apply_distinct_on_objects_group(groups: &mut Vec<Vec<GQLObject>>, hidden_selections: &[String]) {
@@ -147,7 +171,7 @@ fn apply_distinct_on_objects_group(groups: &mut Vec<Vec<GQLObject>>, hidden_sele
         // Build row of the selected only values
         let mut row_values: Vec<String> = Vec::with_capacity(titles_count);
         for key in &titles {
-            row_values.push(object.attributes.get(key as &str).unwrap().literal());
+            row_values.push(object.attributes.get(key as &str).unwrap().to_string());
         }
 
         // Compute the hash for row of values
