@@ -1,7 +1,9 @@
 use gitql_ast::date_utils::date_time_to_time_stamp;
 use gitql_ast::date_utils::date_to_time_stamp;
+use gitql_ast::environment::Environment;
 use gitql_ast::expression::ArithmeticExpression;
 use gitql_ast::expression::ArithmeticOperator;
+use gitql_ast::expression::AssignmentExpression;
 use gitql_ast::expression::BetweenExpression;
 use gitql_ast::expression::BitwiseExpression;
 use gitql_ast::expression::BitwiseOperator;
@@ -13,6 +15,7 @@ use gitql_ast::expression::ComparisonOperator;
 use gitql_ast::expression::Expression;
 use gitql_ast::expression::ExpressionKind::*;
 use gitql_ast::expression::GlobExpression;
+use gitql_ast::expression::GlobalVariableExpression;
 use gitql_ast::expression::InExpression;
 use gitql_ast::expression::IsNullExpression;
 use gitql_ast::expression::LikeExpression;
@@ -34,10 +37,18 @@ use std::string::String;
 
 #[allow(clippy::borrowed_box)]
 pub fn evaluate_expression(
+    env: &mut Environment,
     expression: &Box<dyn Expression>,
     object: &HashMap<String, Value>,
 ) -> Result<Value, String> {
-    match expression.expression_kind() {
+    match expression.kind() {
+        Assignment => {
+            let expr = expression
+                .as_any()
+                .downcast_ref::<AssignmentExpression>()
+                .unwrap();
+            evaluate_assignment(env, expr, object)
+        }
         String => {
             let expr = expression
                 .as_any()
@@ -52,6 +63,13 @@ pub fn evaluate_expression(
                 .unwrap();
             evaluate_symbol(expr, object)
         }
+        GlobalVariable => {
+            let expr = expression
+                .as_any()
+                .downcast_ref::<GlobalVariableExpression>()
+                .unwrap();
+            evaluate_global_variable(env, expr)
+        }
         Number => {
             let expr = expression
                 .as_any()
@@ -68,86 +86,96 @@ pub fn evaluate_expression(
         }
         PrefixUnary => {
             let expr = expression.as_any().downcast_ref::<PrefixUnary>().unwrap();
-            evaluate_prefix_unary(expr, object)
+            evaluate_prefix_unary(env, expr, object)
         }
         Arithmetic => {
             let expr = expression
                 .as_any()
                 .downcast_ref::<ArithmeticExpression>()
                 .unwrap();
-            evaluate_arithmetic(expr, object)
+            evaluate_arithmetic(env, expr, object)
         }
         Comparison => {
             let expr = expression
                 .as_any()
                 .downcast_ref::<ComparisonExpression>()
                 .unwrap();
-            evaluate_comparison(expr, object)
+            evaluate_comparison(env, expr, object)
         }
         Like => {
             let expr = expression
                 .as_any()
                 .downcast_ref::<LikeExpression>()
                 .unwrap();
-            evaulate_like(expr, object)
+            evaluate_like(env, expr, object)
         }
         Glob => {
             let expr = expression
                 .as_any()
                 .downcast_ref::<GlobExpression>()
                 .unwrap();
-            evaulate_glob(expr, object)
+            evaluate_glob(env, expr, object)
         }
         Logical => {
             let expr = expression
                 .as_any()
                 .downcast_ref::<LogicalExpression>()
                 .unwrap();
-            evaluate_logical(expr, object)
+            evaluate_logical(env, expr, object)
         }
         Bitwise => {
             let expr = expression
                 .as_any()
                 .downcast_ref::<BitwiseExpression>()
                 .unwrap();
-            evaluate_bitwise(expr, object)
+            evaluate_bitwise(env, expr, object)
         }
         Call => {
             let expr = expression
                 .as_any()
                 .downcast_ref::<CallExpression>()
                 .unwrap();
-            evaluate_call(expr, object)
+            evaluate_call(env, expr, object)
         }
         Between => {
             let expr = expression
                 .as_any()
                 .downcast_ref::<BetweenExpression>()
                 .unwrap();
-            evaluate_between(expr, object)
+            evaluate_between(env, expr, object)
         }
         Case => {
             let expr = expression
                 .as_any()
                 .downcast_ref::<CaseExpression>()
                 .unwrap();
-            evaluate_case(expr, object)
+            evaluate_case(env, expr, object)
         }
         In => {
             let expr = expression.as_any().downcast_ref::<InExpression>().unwrap();
-            evaluate_in(expr, object)
+            evaluate_in(env, expr, object)
         }
         IsNull => {
             let expr = expression
                 .as_any()
                 .downcast_ref::<IsNullExpression>()
                 .unwrap();
-            evaluate_is_null(expr, object)
+            evaluate_is_null(env, expr, object)
         }
         Null => Ok(Value::Null),
     }
 }
 
+fn evaluate_assignment(
+    env: &mut Environment,
+    expr: &AssignmentExpression,
+    object: &HashMap<String, Value>,
+) -> Result<Value, String> {
+    let value = evaluate_expression(env, &expr.value, object)?;
+    env.globals.insert(expr.symbol.to_string(), value.clone());
+    Ok(value)
+}
+
 fn evaluate_string(expr: &StringExpression) -> Result<Value, String> {
     match expr.value_type {
         StringValueType::Text => Ok(Value::Text(expr.value.to_owned())),
@@ -173,6 +201,21 @@ fn evaluate_symbol(
     Err(format!("Invalid column name `{}`", &expr.value))
 }
 
+fn evaluate_global_variable(
+    env: &mut Environment,
+    expr: &GlobalVariableExpression,
+) -> Result<Value, String> {
+    let name = &expr.name;
+    if env.globals.contains_key(name) {
+        return Ok(env.globals[name].clone());
+    }
+
+    Err(format!(
+        "The value of `{}` may be not exists or calculated yet",
+        name
+    ))
+}
+
 fn evaluate_number(expr: &NumberExpression) -> Result<Value, String> {
     Ok(expr.value.to_owned())
 }
@@ -182,10 +225,11 @@ fn evaluate_boolean(expr: &BooleanExpression) -> Result<Value, String> {
 }
 
 fn evaluate_prefix_unary(
+    env: &mut Environment,
     expr: &PrefixUnary,
     object: &HashMap<String, Value>,
 ) -> Result<Value, String> {
-    let rhs = evaluate_expression(&expr.right, object)?;
+    let rhs = evaluate_expression(env, &expr.right, object)?;
     if expr.op == PrefixUnaryOperator::Bang {
         Ok(Value::Boolean(!rhs.as_bool()))
     } else {
@@ -194,11 +238,12 @@ fn evaluate_prefix_unary(
 }
 
 fn evaluate_arithmetic(
+    env: &mut Environment,
     expr: &ArithmeticExpression,
     object: &HashMap<String, Value>,
 ) -> Result<Value, String> {
-    let lhs = evaluate_expression(&expr.left, object)?;
-    let rhs = evaluate_expression(&expr.right, object)?;
+    let lhs = evaluate_expression(env, &expr.left, object)?;
+    let rhs = evaluate_expression(env, &expr.right, object)?;
 
     match expr.operator {
         ArithmeticOperator::Plus => Ok(lhs.plus(&rhs)),
@@ -210,11 +255,12 @@ fn evaluate_arithmetic(
 }
 
 fn evaluate_comparison(
+    env: &mut Environment,
     expr: &ComparisonExpression,
     object: &HashMap<String, Value>,
 ) -> Result<Value, String> {
-    let lhs = evaluate_expression(&expr.left, object)?;
-    let rhs = evaluate_expression(&expr.right, object)?;
+    let lhs = evaluate_expression(env, &expr.left, object)?;
+    let rhs = evaluate_expression(env, &expr.right, object)?;
 
     let left_type = lhs.data_type();
     let comparison_result = if left_type == DataType::Integer {
@@ -230,7 +276,7 @@ fn evaluate_comparison(
         let irhs = rhs.as_bool();
         ilhs.cmp(&irhs)
     } else {
-        lhs.literal().cmp(&rhs.literal())
+        lhs.to_string().cmp(&rhs.to_string())
     };
 
     if expr.operator == ComparisonOperator::NullSafeEqual {
@@ -265,8 +311,12 @@ fn evaluate_comparison(
     }))
 }
 
-fn evaulate_like(expr: &LikeExpression, object: &HashMap<String, Value>) -> Result<Value, String> {
-    let rhs = evaluate_expression(&expr.pattern, object)?.as_text();
+fn evaluate_like(
+    env: &mut Environment,
+    expr: &LikeExpression,
+    object: &HashMap<String, Value>,
+) -> Result<Value, String> {
+    let rhs = evaluate_expression(env, &expr.pattern, object)?.as_text();
     let pattern = &format!(
         "^{}$",
         rhs.to_lowercase().replace('%', ".*").replace('_', ".")
@@ -276,14 +326,18 @@ fn evaulate_like(expr: &LikeExpression, object: &HashMap<String, Value>) -> Resu
         return Err(regex_result.err().unwrap().to_string());
     }
     let regex = regex_result.ok().unwrap();
-    let lhs = evaluate_expression(&expr.input, object)?
+    let lhs = evaluate_expression(env, &expr.input, object)?
         .as_text()
         .to_lowercase();
     Ok(Value::Boolean(regex.is_match(&lhs)))
 }
 
-fn evaulate_glob(expr: &GlobExpression, object: &HashMap<String, Value>) -> Result<Value, String> {
-    let rhs = evaluate_expression(&expr.pattern, object)?.as_text();
+fn evaluate_glob(
+    env: &mut Environment,
+    expr: &GlobExpression,
+    object: &HashMap<String, Value>,
+) -> Result<Value, String> {
+    let rhs = evaluate_expression(env, &expr.pattern, object)?.as_text();
     let pattern = &format!(
         "^{}$",
         rhs.replace('.', "\\.").replace('*', ".*").replace('?', ".")
@@ -293,15 +347,16 @@ fn evaulate_glob(expr: &GlobExpression, object: &HashMap<String, Value>) -> Resu
         return Err(regex_result.err().unwrap().to_string());
     }
     let regex = regex_result.ok().unwrap();
-    let lhs = evaluate_expression(&expr.input, object)?.as_text();
+    let lhs = evaluate_expression(env, &expr.input, object)?.as_text();
     Ok(Value::Boolean(regex.is_match(&lhs)))
 }
 
 fn evaluate_logical(
+    env: &mut Environment,
     expr: &LogicalExpression,
     object: &HashMap<String, Value>,
 ) -> Result<Value, String> {
-    let lhs = evaluate_expression(&expr.left, object)?.as_bool();
+    let lhs = evaluate_expression(env, &expr.left, object)?.as_bool();
     if expr.operator == LogicalOperator::And && !lhs {
         return Ok(Value::Boolean(false));
     }
@@ -310,7 +365,7 @@ fn evaluate_logical(
         return Ok(Value::Boolean(true));
     }
 
-    let rhs = evaluate_expression(&expr.right, object)?.as_bool();
+    let rhs = evaluate_expression(env, &expr.right, object)?.as_bool();
 
     Ok(Value::Boolean(match expr.operator {
         LogicalOperator::And => lhs && rhs,
@@ -320,11 +375,12 @@ fn evaluate_logical(
 }
 
 fn evaluate_bitwise(
+    env: &mut Environment,
     expr: &BitwiseExpression,
     object: &HashMap<String, Value>,
 ) -> Result<Value, String> {
-    let lhs = evaluate_expression(&expr.left, object)?.as_int();
-    let rhs = evaluate_expression(&expr.right, object)?.as_int();
+    let lhs = evaluate_expression(env, &expr.left, object)?.as_int();
+    let rhs = evaluate_expression(env, &expr.right, object)?.as_int();
 
     match expr.operator {
         BitwiseOperator::Or => Ok(Value::Integer(lhs | rhs)),
@@ -346,52 +402,65 @@ fn evaluate_bitwise(
     }
 }
 
-fn evaluate_call(expr: &CallExpression, object: &HashMap<String, Value>) -> Result<Value, String> {
+fn evaluate_call(
+    env: &mut Environment,
+    expr: &CallExpression,
+    object: &HashMap<String, Value>,
+) -> Result<Value, String> {
     let function_name = expr.function_name.as_str();
     let function = FUNCTIONS.get(function_name).unwrap();
 
     let mut arguments = vec![];
     for arg in expr.arguments.iter() {
-        arguments.push(evaluate_expression(arg, object)?);
+        arguments.push(evaluate_expression(env, arg, object)?);
     }
 
-    Ok(function(arguments))
+    Ok(function(&arguments))
 }
 
 fn evaluate_between(
+    env: &mut Environment,
     expr: &BetweenExpression,
     object: &HashMap<String, Value>,
 ) -> Result<Value, String> {
-    let value_result = evaluate_expression(&expr.value, object)?;
-    let range_start_result = evaluate_expression(&expr.range_start, object)?;
-    let range_end_result = evaluate_expression(&expr.range_end, object)?;
+    let value_result = evaluate_expression(env, &expr.value, object)?;
+    let range_start_result = evaluate_expression(env, &expr.range_start, object)?;
+    let range_end_result = evaluate_expression(env, &expr.range_end, object)?;
     let value = value_result.as_int();
     let range_start = range_start_result.as_int();
     let range_end = range_end_result.as_int();
     Ok(Value::Boolean(value >= range_start && value <= range_end))
 }
 
-fn evaluate_case(expr: &CaseExpression, object: &HashMap<String, Value>) -> Result<Value, String> {
+fn evaluate_case(
+    env: &mut Environment,
+    expr: &CaseExpression,
+    object: &HashMap<String, Value>,
+) -> Result<Value, String> {
     let conditions = &expr.conditions;
     let values = &expr.values;
 
     for i in 0..conditions.len() {
-        let condition = evaluate_expression(&conditions[i], object)?;
+        let condition = evaluate_expression(env, &conditions[i], object)?;
         if condition.as_bool() {
-            return evaluate_expression(&values[i], object);
+            return evaluate_expression(env, &values[i], object);
         }
     }
 
     match &expr.default_value {
-        Some(default_value) => evaluate_expression(default_value, object),
+        Some(default_value) => evaluate_expression(env, default_value, object),
         _ => Err("Invalid case statement".to_owned()),
     }
 }
 
-fn evaluate_in(expr: &InExpression, object: &HashMap<String, Value>) -> Result<Value, String> {
-    let argument = evaluate_expression(&expr.argument, object)?;
+fn evaluate_in(
+    env: &mut Environment,
+    expr: &InExpression,
+    object: &HashMap<String, Value>,
+) -> Result<Value, String> {
+    let argument = evaluate_expression(env, &expr.argument, object)?;
     for value_expr in &expr.values {
-        let value = evaluate_expression(value_expr, object)?;
+        let value = evaluate_expression(env, value_expr, object)?;
         if argument.equals(&value) {
             return Ok(Value::Boolean(true));
         }
@@ -400,11 +469,12 @@ fn evaluate_in(expr: &InExpression, object: &HashMap<String, Value>) -> Result<V
 }
 
 fn evaluate_is_null(
+    env: &mut Environment,
     expr: &IsNullExpression,
     object: &HashMap<String, Value>,
 ) -> Result<Value, String> {
-    let argument = evaluate_expression(&expr.argument, object)?;
-    let is_null = argument.data_type().is_type(DataType::Null);
+    let argument = evaluate_expression(env, &expr.argument, object)?;
+    let is_null = argument.data_type().is_null();
     Ok(Value::Boolean(if expr.has_not {
         !is_null
     } else {
