@@ -1,3 +1,5 @@
+use gitql_ast::environment::Environment;
+use gix::reference::Category;
 use std::collections::HashMap;
 
 use gitql_ast::expression::Expression;
@@ -8,24 +10,26 @@ use gitql_ast::value::Value;
 use crate::engine_evaluator::evaluate_expression;
 
 pub fn select_gql_objects(
-    repo: &git2::Repository,
+    env: &mut Environment,
+    repo: &gix::Repository,
     table: String,
     fields_names: &Vec<String>,
     fields_values: &Vec<Box<dyn Expression>>,
     alias_table: &HashMap<String, String>,
 ) -> Result<Vec<GQLObject>, String> {
     match table.as_str() {
-        "refs" => select_references(repo, fields_names, fields_values, alias_table),
-        "commits" => select_commits(repo, fields_names, fields_values, alias_table),
-        "branches" => select_branches(repo, fields_names, fields_values, alias_table),
-        "diffs" => select_diffs(repo, fields_names, fields_values, alias_table),
-        "tags" => select_tags(repo, fields_names, fields_values, alias_table),
-        _ => select_values(repo, fields_names, fields_values, alias_table),
+        "refs" => select_references(env, repo, fields_names, fields_values, alias_table),
+        "commits" => select_commits(env, repo, fields_names, fields_values, alias_table),
+        "branches" => select_branches(env, repo, fields_names, fields_values, alias_table),
+        "diffs" => select_diffs(env, repo, fields_names, fields_values, alias_table),
+        "tags" => select_tags(env, repo, fields_names, fields_values, alias_table),
+        _ => select_values(env, fields_names, fields_values, alias_table),
     }
 }
 
 fn select_references(
-    repo: &git2::Repository,
+    env: &mut Environment,
+    repo: &gix::Repository,
     fields_names: &Vec<String>,
     fields_values: &Vec<Box<dyn Expression>>,
     alias_table: &HashMap<String, String>,
@@ -42,12 +46,7 @@ fn select_references(
     let values_len = fields_values.len() as i64;
     let padding = names_len - values_len;
 
-    for reference_result in references {
-        if reference_result.is_err() {
-            break;
-        }
-
-        let reference = reference_result.ok().unwrap();
+    for reference in references.all().unwrap().flatten() {
         let mut attributes: HashMap<String, Value> = HashMap::with_capacity(fields_names.len());
 
         for index in 0..names_len {
@@ -56,36 +55,42 @@ fn select_references(
             if (index - padding) >= 0 {
                 let value = &fields_values[(index - padding) as usize];
                 if value.as_any().downcast_ref::<SymbolExpression>().is_none() {
-                    let evaulated = evaluate_expression(value, &attributes)?;
+                    let evaluated = evaluate_expression(env, value, &attributes)?;
                     let column_name = get_column_name(alias_table, field_name);
-                    attributes.insert(column_name, evaulated);
+                    attributes.insert(column_name, evaluated);
                     continue;
                 }
             }
 
             if field_name == "name" {
-                let name = reference.shorthand().unwrap_or("").to_string();
-                let column_name = get_column_name(alias_table, &"name".to_string());
+                let name = reference
+                    .name()
+                    .category_and_short_name()
+                    .map(|(_, sn)| sn)
+                    .unwrap_or("".into())
+                    .to_string();
+                let column_name = get_column_name(alias_table, "name");
                 attributes.insert(column_name, Value::Text(name));
                 continue;
             }
 
             if field_name == "full_name" {
-                let full_name = reference.name().unwrap_or("").to_string();
-                let column_name = get_column_name(alias_table, &"full_name".to_string());
+                let full_name = reference.name().as_bstr().to_string();
+                let column_name = get_column_name(alias_table, "full_name");
                 attributes.insert(column_name, Value::Text(full_name));
                 continue;
             }
 
             if field_name == "type" {
-                let column_name = get_column_name(alias_table, &"type".to_string());
-                if reference.is_branch() {
+                let column_name = get_column_name(alias_table, "type");
+                let category = reference.name().category();
+                if category.map_or(false, |cat| cat == Category::LocalBranch) {
                     attributes.insert(column_name, Value::Text("branch".to_owned()));
-                } else if reference.is_remote() {
+                } else if category.map_or(false, |cat| cat == Category::RemoteBranch) {
                     attributes.insert(column_name, Value::Text("remote".to_owned()));
-                } else if reference.is_tag() {
+                } else if category.map_or(false, |cat| cat == Category::Tag) {
                     attributes.insert(column_name, Value::Text("tag".to_owned()));
-                } else if reference.is_note() {
+                } else if category.map_or(false, |cat| cat == Category::Note) {
                     attributes.insert(column_name, Value::Text("note".to_owned()));
                 } else {
                     attributes.insert(column_name, Value::Text("other".to_owned()));
@@ -94,7 +99,7 @@ fn select_references(
             }
 
             if field_name == "repo" {
-                let column_name = get_column_name(alias_table, &"repo".to_string());
+                let column_name = get_column_name(alias_table, "repo");
                 attributes.insert(column_name, Value::Text(repo_path.to_string()));
                 continue;
             }
@@ -108,7 +113,8 @@ fn select_references(
 }
 
 fn select_commits(
-    repo: &git2::Repository,
+    env: &mut Environment,
+    repo: &gix::Repository,
     fields_names: &Vec<String>,
     fields_values: &Vec<Box<dyn Expression>>,
     alias_table: &HashMap<String, String>,
@@ -116,15 +122,21 @@ fn select_commits(
     let repo_path = repo.path().to_str().unwrap().to_string();
 
     let mut commits: Vec<GQLObject> = Vec::new();
-    let mut revwalk = repo.revwalk().unwrap();
-    revwalk.push_head().unwrap();
+    let head_id = repo.head_id();
+    if head_id.is_err() {
+        return Ok(commits);
+    }
+
+    let revwalk = head_id.unwrap().ancestors().all().unwrap();
 
     let names_len = fields_names.len() as i64;
     let values_len = fields_values.len() as i64;
     let padding = names_len - values_len;
 
-    for commit_id in revwalk {
-        let commit = repo.find_commit(commit_id.unwrap()).unwrap();
+    for commit_info in revwalk {
+        let commit_info = commit_info.unwrap();
+        let commit = repo.find_object(commit_info.id).unwrap().into_commit();
+        let commit = commit.decode().unwrap();
 
         let mut attributes: HashMap<String, Value> = HashMap::with_capacity(fields_names.len());
 
@@ -134,57 +146,59 @@ fn select_commits(
             if (index - padding) >= 0 {
                 let value = &fields_values[(index - padding) as usize];
                 if value.as_any().downcast_ref::<SymbolExpression>().is_none() {
-                    let evaulated = evaluate_expression(value, &attributes)?;
+                    let evaluated = evaluate_expression(env, value, &attributes)?;
                     let column_name = get_column_name(alias_table, field_name);
-                    attributes.insert(column_name, evaulated);
+                    attributes.insert(column_name, evaluated);
                     continue;
                 }
             }
 
             if field_name == "commit_id" {
-                let commit_id = Value::Text(commit.id().to_string());
-                let column_name = get_column_name(alias_table, &"commit_id".to_string());
+                let commit_id = Value::Text(commit_info.id.to_string());
+                let column_name = get_column_name(alias_table, "commit_id");
                 attributes.insert(column_name, commit_id);
                 continue;
             }
 
             if field_name == "name" {
-                let name = commit.author().name().unwrap_or("").to_string();
-                let column_name = get_column_name(alias_table, &"name".to_string());
+                let name = commit.author().name.to_string();
+                let column_name = get_column_name(alias_table, "name");
                 attributes.insert(column_name, Value::Text(name));
                 continue;
             }
 
             if field_name == "email" {
-                let email = commit.author().email().unwrap_or("").to_string();
-                let column_name = get_column_name(alias_table, &"email".to_string());
+                let email = commit.author().email.to_string();
+                let column_name = get_column_name(alias_table, "email");
                 attributes.insert(column_name, Value::Text(email));
                 continue;
             }
 
             if field_name == "title" {
-                let summary = Value::Text(commit.summary().unwrap().to_string());
-                let column_name = get_column_name(alias_table, &"title".to_string());
+                let summary = Value::Text(commit.message().summary().to_string());
+                let column_name = get_column_name(alias_table, "title");
                 attributes.insert(column_name, summary);
                 continue;
             }
 
             if field_name == "message" {
-                let message = Value::Text(commit.message().unwrap_or("").to_string());
-                let column_name = get_column_name(alias_table, &"message".to_string());
+                let message = Value::Text(commit.message.to_string());
+                let column_name = get_column_name(alias_table, "message");
                 attributes.insert(column_name, message);
                 continue;
             }
 
             if field_name == "datetime" {
-                let column_name = get_column_name(alias_table, &"datetime".to_string());
-                let time_stamp = commit.time().seconds();
+                let column_name = get_column_name(alias_table, "datetime");
+                let time_stamp = commit_info
+                    .commit_time
+                    .unwrap_or_else(|| commit.time().seconds);
                 attributes.insert(column_name, Value::DateTime(time_stamp));
                 continue;
             }
 
             if field_name == "repo" {
-                let column_name = get_column_name(alias_table, &"repo".to_string());
+                let column_name = get_column_name(alias_table, "repo");
                 attributes.insert(column_name, Value::Text(repo_path.to_string()));
                 continue;
             }
@@ -198,23 +212,34 @@ fn select_commits(
 }
 
 fn select_diffs(
-    repo: &git2::Repository,
+    env: &mut Environment,
+    repo: &gix::Repository,
     fields_names: &Vec<String>,
     fields_values: &Vec<Box<dyn Expression>>,
     alias_table: &HashMap<String, String>,
 ) -> Result<Vec<GQLObject>, String> {
-    let mut diffs: Vec<GQLObject> = Vec::new();
-    let mut revwalk = repo.revwalk().unwrap();
-    revwalk.push_head().unwrap();
+    let repo = {
+        let mut repo = repo.clone();
+        repo.object_cache_size_if_unset(4 * 1024 * 1024);
+        repo
+    };
 
+    let mut diffs: Vec<GQLObject> = Vec::new();
+    let revwalk = repo.head_id().unwrap().ancestors().all().unwrap();
     let repo_path = repo.path().to_str().unwrap().to_string();
 
+    let mut rewrite_cache = repo
+        .diff_resource_cache(gix::diff::blob::pipeline::Mode::ToGit, Default::default())
+        .unwrap();
+    let mut diff_cache = rewrite_cache.clone();
+
     let names_len = fields_names.len() as i64;
     let values_len = fields_values.len() as i64;
     let padding = names_len - values_len;
 
-    for commit_id in revwalk {
-        let commit = repo.find_commit(commit_id.unwrap()).unwrap();
+    for commit_info in revwalk {
+        let commit_info = commit_info.unwrap();
+        let commit = commit_info.id().object().unwrap().into_commit();
         let mut attributes: HashMap<String, Value> = HashMap::with_capacity(fields_names.len());
 
         for index in 0..names_len {
@@ -223,35 +248,35 @@ fn select_diffs(
             if (index - padding) >= 0 {
                 let value = &fields_values[(index - padding) as usize];
                 if value.as_any().downcast_ref::<SymbolExpression>().is_none() {
-                    let evaulated = evaluate_expression(value, &attributes)?;
+                    let evaluated = evaluate_expression(env, value, &attributes)?;
                     let column_name = get_column_name(alias_table, field_name);
-                    attributes.insert(column_name, evaulated);
+                    attributes.insert(column_name, evaluated);
                     continue;
                 }
             }
 
             if field_name == "commit_id" {
-                let column_name = get_column_name(alias_table, &"commit_id".to_string());
-                attributes.insert(column_name, Value::Text(commit.id().to_string()));
+                let column_name = get_column_name(alias_table, "commit_id");
+                attributes.insert(column_name, Value::Text(commit_info.id.to_string()));
                 continue;
             }
 
             if field_name == "name" {
-                let name = commit.author().name().unwrap_or("").to_string();
-                let column_name = get_column_name(alias_table, &"name".to_string());
+                let name = commit.author().unwrap().name.to_string();
+                let column_name = get_column_name(alias_table, "name");
                 attributes.insert(column_name, Value::Text(name));
                 continue;
             }
 
             if field_name == "email" {
-                let email = commit.author().email().unwrap_or("").to_string();
-                let column_name = get_column_name(alias_table, &"email".to_string());
+                let email = commit.author().unwrap().email.to_string();
+                let column_name = get_column_name(alias_table, "email");
                 attributes.insert(column_name, Value::Text(email));
                 continue;
             }
 
             if field_name == "repo" {
-                let column_name = get_column_name(alias_table, &"repo".to_string());
+                let column_name = get_column_name(alias_table, "repo");
                 attributes.insert(column_name, Value::Text(repo_path.to_string()));
                 continue;
             }
@@ -260,36 +285,57 @@ fn select_diffs(
                 || field_name == "deletions"
                 || field_name == "files_changed"
             {
-                let diff = if commit.parents().len() > 0 {
-                    repo.diff_tree_to_tree(
-                        Some(&commit.parent(0).unwrap().tree().unwrap()),
-                        Some(&commit.tree().unwrap()),
-                        None,
-                    )
-                } else {
-                    repo.diff_tree_to_tree(None, Some(&commit.tree().unwrap()), None)
-                };
+                let current = commit.tree().unwrap();
+                let previous = commit_info
+                    .parent_ids()
+                    .next()
+                    .map(|id| id.object().unwrap().into_commit().tree().unwrap())
+                    .unwrap_or_else(|| repo.empty_tree());
 
-                let diff_status = diff.unwrap().stats().unwrap();
+                let select_insertions_or_deletions =
+                    field_name == "insertions" || field_name == "deletions";
+
+                rewrite_cache.clear_resource_cache();
+                diff_cache.clear_resource_cache();
+
+                let (mut insertions, mut deletions, mut files_changed) = (0, 0, 0);
+
+                previous
+                    .changes()
+                    .unwrap()
+                    .for_each_to_obtain_tree_with_cache(
+                        &current,
+                        &mut rewrite_cache,
+                        |change| -> Result<_, gix::object::blob::diff::init::Error> {
+                            files_changed += usize::from(change.event.entry_mode().is_no_tree());
+                            if select_insertions_or_deletions {
+                                if let Ok(mut platform) = change.diff(&mut diff_cache) {
+                                    if let Ok(Some(counts)) = platform.line_counts() {
+                                        deletions += counts.removals;
+                                        insertions += counts.insertions;
+                                    }
+                                }
+                            }
+                            Ok(gix::object::tree::diff::Action::Continue)
+                        },
+                    )
+                    .unwrap();
 
                 if field_name == "insertions" {
-                    let insertions = Value::Integer(diff_status.insertions() as i64);
-                    let column_name = get_column_name(alias_table, &"insertions".to_string());
-                    attributes.insert(column_name, insertions);
+                    let column_name = get_column_name(alias_table, "insertions");
+                    attributes.insert(column_name, Value::Integer(insertions as i64));
                     continue;
                 }
 
                 if field_name == "deletions" {
-                    let deletations = Value::Integer(diff_status.deletions() as i64);
-                    let column_name = get_column_name(alias_table, &"deletions".to_string());
-                    attributes.insert(column_name, deletations);
+                    let column_name = get_column_name(alias_table, "deletions");
+                    attributes.insert(column_name, Value::Integer(deletions as i64));
                     continue;
                 }
 
                 if field_name == "files_changed" {
-                    let file_changed = Value::Integer(diff_status.files_changed() as i64);
-                    let column_name = get_column_name(alias_table, &"files_changed".to_string());
-                    attributes.insert(column_name, file_changed);
+                    let column_name = get_column_name(alias_table, "files_changed");
+                    attributes.insert(column_name, Value::Integer(files_changed as i64));
                     continue;
                 }
             }
@@ -303,22 +349,35 @@ fn select_diffs(
 }
 
 fn select_branches(
-    repo: &git2::Repository,
+    env: &mut Environment,
+    repo: &gix::Repository,
     fields_names: &Vec<String>,
     fields_values: &Vec<Box<dyn Expression>>,
     alias_table: &HashMap<String, String>,
 ) -> Result<Vec<GQLObject>, String> {
     let mut branches: Vec<GQLObject> = Vec::new();
-    let local_and_remote_branches = repo.branches(None).unwrap();
     let repo_path = repo.path().to_str().unwrap().to_string();
+    let platform = repo.references().unwrap();
+    let local_branches = platform.local_branches().unwrap();
+    let remote_branches = platform.remote_branches().unwrap();
+    let local_and_remote_branches = local_branches.chain(remote_branches);
+    let head_ref_result = repo.head_ref();
+    if head_ref_result.is_err() {
+        return Ok(branches);
+    }
+
+    let head_ref_option = head_ref_result.unwrap();
+    if head_ref_option.is_none() {
+        return Ok(branches);
+    }
+
+    let head_ref = head_ref_option.unwrap();
 
     let names_len = fields_names.len() as i64;
     let values_len = fields_values.len() as i64;
     let padding = names_len - values_len;
 
-    for branch in local_and_remote_branches {
-        let (branch, _) = branch.unwrap();
-
+    for branch in local_and_remote_branches.flatten() {
         let mut attributes: HashMap<String, Value> = HashMap::with_capacity(fields_names.len());
         for index in 0..names_len {
             let field_name = &fields_names[index as usize];
@@ -326,43 +385,53 @@ fn select_branches(
             if (index - padding) >= 0 {
                 let value = &fields_values[(index - padding) as usize];
                 if value.as_any().downcast_ref::<SymbolExpression>().is_none() {
-                    let evaulated = evaluate_expression(value, &attributes)?;
+                    let evaluated = evaluate_expression(env, value, &attributes)?;
                     let column_name = get_column_name(alias_table, field_name);
-                    attributes.insert(column_name, evaulated);
+                    attributes.insert(column_name, evaluated);
                     continue;
                 }
             }
 
             if field_name == "name" {
-                let branch_name = branch.name().unwrap().unwrap_or("").to_string();
-                let column_name = get_column_name(alias_table, &"name".to_string());
+                let branch_name = branch.name().as_bstr().to_string();
+                let column_name = get_column_name(alias_table, "name");
                 attributes.insert(column_name, Value::Text(branch_name));
                 continue;
             }
 
             if field_name == "commit_count" {
-                let branch_ref = branch.get().peel_to_commit().unwrap();
-                let mut revwalk = repo.revwalk().unwrap();
-                let _ = revwalk.push(branch_ref.id());
-                let column_name = get_column_name(alias_table, &"commit_count".to_string());
-                attributes.insert(column_name, Value::Integer(revwalk.count() as i64));
+                let commit_count = if let Some(id) = branch.try_id() {
+                    if let Ok(revwalk) = id.ancestors().all() {
+                        revwalk.count() as i64
+                    } else {
+                        -1
+                    }
+                } else {
+                    -1
+                };
+                let column_name = get_column_name(alias_table, "commit_count");
+                attributes.insert(column_name, Value::Integer(commit_count));
                 continue;
             }
 
             if field_name == "is_head" {
-                let column_name = get_column_name(alias_table, &"is_head".to_string());
-                attributes.insert(column_name, Value::Boolean(branch.is_head()));
+                let column_name = get_column_name(alias_table, "is_head");
+                attributes.insert(column_name, Value::Boolean(branch.inner == head_ref.inner));
                 continue;
             }
 
             if field_name == "is_remote" {
-                let column_name = get_column_name(alias_table, &"is_remote".to_string());
-                attributes.insert(column_name, Value::Boolean(branch.get().is_remote()));
+                let column_name = get_column_name(alias_table, "is_remote");
+                let is_remote = branch
+                    .name()
+                    .category()
+                    .map_or(false, |cat| cat == Category::RemoteBranch);
+                attributes.insert(column_name, Value::Boolean(is_remote));
                 continue;
             }
 
             if field_name == "repo" {
-                let column_name = get_column_name(alias_table, &"repo".to_string());
+                let column_name = get_column_name(alias_table, "repo");
                 attributes.insert(column_name, Value::Text(repo_path.to_string()));
                 continue;
             }
@@ -376,20 +445,22 @@ fn select_branches(
 }
 
 fn select_tags(
-    repo: &git2::Repository,
+    env: &mut Environment,
+    repo: &gix::Repository,
     fields_names: &Vec<String>,
     fields_values: &Vec<Box<dyn Expression>>,
     alias_table: &HashMap<String, String>,
 ) -> Result<Vec<GQLObject>, String> {
     let mut tags: Vec<GQLObject> = Vec::new();
-    let tag_names = repo.tag_names(None).unwrap();
+    let platform = repo.references().unwrap();
+    let tag_names = platform.tags().unwrap();
     let repo_path = repo.path().to_str().unwrap().to_string();
 
     let names_len = fields_names.len() as i64;
     let values_len = fields_values.len() as i64;
     let padding = names_len - values_len;
 
-    for tag_name in tag_names.iter().flatten() {
+    for tag_ref in tag_names.flatten() {
         let mut attributes: HashMap<String, Value> = HashMap::with_capacity(fields_names.len());
 
         for index in 0..names_len {
@@ -398,21 +469,25 @@ fn select_tags(
                 let value = &fields_values[(index - padding) as usize];
 
                 if value.as_any().downcast_ref::<SymbolExpression>().is_none() {
-                    let evaulated = evaluate_expression(value, &attributes)?;
+                    let evaluated = evaluate_expression(env, value, &attributes)?;
                     let column_name = get_column_name(alias_table, field_name);
-                    attributes.insert(column_name, evaulated);
+                    attributes.insert(column_name, evaluated);
                     continue;
                 }
             }
 
             if field_name == "name" {
-                let column_name = get_column_name(alias_table, &"name".to_string());
+                let column_name = get_column_name(alias_table, "name");
+                let tag_name = tag_ref
+                    .name()
+                    .category_and_short_name()
+                    .map_or_else(String::default, |(_, short_name)| short_name.to_string());
                 attributes.insert(column_name, Value::Text(tag_name.to_string()));
                 continue;
             }
 
             if field_name == "repo" {
-                let column_name = get_column_name(alias_table, &"repo".to_string());
+                let column_name = get_column_name(alias_table, "repo");
                 attributes.insert(column_name, Value::Text(repo_path.to_string()));
                 continue;
             }
@@ -426,30 +501,27 @@ fn select_tags(
 }
 
 fn select_values(
-    _repo: &git2::Repository,
+    env: &mut Environment,
     fields_names: &[String],
     fields_values: &Vec<Box<dyn Expression>>,
     alias_table: &HashMap<String, String>,
 ) -> Result<Vec<GQLObject>, String> {
-    let mut values: Vec<GQLObject> = Vec::new();
-    let mut attributes: HashMap<String, Value> = HashMap::new();
     let len = fields_values.len();
+    let mut attributes: HashMap<String, Value> = HashMap::with_capacity(len);
 
     for index in 0..len {
         let field_name = &fields_names[index];
         let value = &fields_values[index];
-        let evaulated = evaluate_expression(value, &attributes)?;
+        let evaluated = evaluate_expression(env, value, &attributes)?;
         let column_name = get_column_name(alias_table, field_name);
-        attributes.insert(column_name, evaulated);
+        attributes.insert(column_name, evaluated);
     }
 
-    let gql_object = GQLObject { attributes };
-    values.push(gql_object);
-    Ok(values)
+    Ok(vec![GQLObject { attributes }])
 }
 
 #[inline(always)]
-pub fn get_column_name(alias_table: &HashMap<String, String>, name: &String) -> String {
+pub fn get_column_name(alias_table: &HashMap<String, String>, name: &str) -> String {
     alias_table
         .get(name)
         .unwrap_or(&name.to_string())
